@@ -1309,6 +1309,8 @@ _partition_check_basic_sanity (PedDisk* disk, PedPartition* part)
 {
 	PedPartition*	ext_part = ped_disk_extended_partition (disk);
 
+	PED_ASSERT (part->disk == disk, return 0);
+
 	PED_ASSERT (part->geom.start >= 0, return 0);
 	PED_ASSERT (part->geom.end < disk->dev->length, return 0);
 	PED_ASSERT (part->geom.start <= part->geom.end, return 0);
@@ -1761,6 +1763,10 @@ ped_partition_flag_get_name (PedPartitionFlag flag)
 		return N_("lba");
 	case PED_PARTITION_HPSERVICE:
 		return N_("hp-service");
+	case PED_PARTITION_PALO:
+		return N_("palo");
+	case PED_PARTITION_PREP:
+		return N_("prep-boot");
 
 	default:
 		ped_exception_throw (
