@@ -147,15 +147,15 @@ static int
 check_partition_consistency (const PedDevice* dev,
 	       		     const PC98RawPartition* raw_part)
 {
-	if (raw_part->ipl_sect >= dev->sectors
-	   || raw_part->sector >= dev->sectors
-	   || raw_part->end_sector >= dev->sectors
-	   || raw_part->ipl_head >= dev->heads
-	   || raw_part->head >= dev->heads
-	   || raw_part->end_head >= dev->heads
-	   || PED_LE16_TO_CPU(raw_part->ipl_cyl) >= dev->cylinders
-	   || PED_LE16_TO_CPU(raw_part->cyl) >= dev->cylinders
-	   || PED_LE16_TO_CPU(raw_part->end_cyl) >= dev->cylinders
+	if (raw_part->ipl_sect >= dev->hw_geom.sectors
+	   || raw_part->sector >= dev->hw_geom.sectors
+	   || raw_part->end_sector >= dev->hw_geom.sectors
+	   || raw_part->ipl_head >= dev->hw_geom.heads
+	   || raw_part->head >= dev->hw_geom.heads
+	   || raw_part->end_head >= dev->hw_geom.heads
+	   || PED_LE16_TO_CPU(raw_part->ipl_cyl) >= dev->hw_geom.cylinders
+	   || PED_LE16_TO_CPU(raw_part->cyl) >= dev->hw_geom.cylinders
+	   || PED_LE16_TO_CPU(raw_part->end_cyl) >= dev->hw_geom.cylinders
 	   || PED_LE16_TO_CPU(raw_part->cyl)
 	   	> PED_LE16_TO_CPU(raw_part->end_cyl)
 #if 0
@@ -267,19 +267,24 @@ static PedSector
 chs_to_sector (const PedDevice* dev, int c, int h, int s)
 {
 	PED_ASSERT (dev != NULL, return 0);
-	return (c * dev->heads + h) * dev->sectors + s;
+	return (c * dev->hw_geom.heads + h) * dev->hw_geom.sectors + s;
 }
 
 static void
 sector_to_chs (const PedDevice* dev, PedSector sector, int* c, int* h, int* s)
 {
+	PedSector cyl_size;
+
 	PED_ASSERT (dev != NULL, return);
 	PED_ASSERT (c != NULL, return);
 	PED_ASSERT (h != NULL, return);
 	PED_ASSERT (s != NULL, return);
-	*c = (sector) / (dev->heads * dev->sectors);
-	*h = ((sector) % (dev->heads * dev->sectors)) / dev->sectors;
-	*s = ((sector) % (dev->heads * dev->sectors)) % dev->sectors;
+
+	cyl_size = dev->hw_geom.heads * dev->hw_geom.sectors;
+
+	*c = sector / cyl_size;
+	*h = (sector) % cyl_size / dev->hw_geom.sectors;
+	*s = (sector) % cyl_size % dev->hw_geom.sectors;
 }
 
 static PedSector
@@ -301,8 +306,8 @@ legacy_end (const PedDisk* disk, const PC98RawPartition* raw_part)
 	if (raw_part->end_head == 0 && raw_part->end_sector == 0) {
 		return chs_to_sector (disk->dev,
 				      PED_LE16_TO_CPU(raw_part->end_cyl),
-				      disk->dev->heads - 1,
-				      disk->dev->sectors - 1);
+				      disk->dev->hw_geom.heads - 1,
+				      disk->dev->hw_geom.sectors - 1);
 	} else {
 		return chs_to_sector (disk->dev,
 				      PED_LE16_TO_CPU(raw_part->end_cyl),
@@ -471,8 +476,8 @@ fill_raw_part (PC98RawPartition* raw_part, const PedPartition* part)
 	}
 
 	sector_to_chs (part->disk->dev, part->geom.end, &c, &h, &s);
-	if (h != part->disk->dev->heads - 1
-	    || s != part->disk->dev->sectors - 1) {
+	if (h != part->disk->dev->hw_geom.heads - 1
+	    || s != part->disk->dev->hw_geom.sectors - 1) {
 		ped_exception_throw (
 		    PED_EXCEPTION_NO_FEATURE,
 		    PED_EXCEPTION_CANCEL,
@@ -575,6 +580,7 @@ pc98_partition_duplicate (const PedPartition* part)
 				      part->geom.end);
 	if (!new_part)
 		return NULL;
+	new_part->num = part->num;
 
 	old_pc98_data = (PC98PartitionData*) part->disk_specific;
 	new_pc98_data = (PC98PartitionData*) new_part->disk_specific;
@@ -725,7 +731,7 @@ _primary_constraint (PedDisk* disk)
 	PedGeometry	max_geom;
 	PedSector	cylinder_size;
 
-	cylinder_size = dev->sectors * dev->heads;
+	cylinder_size = dev->hw_geom.sectors * dev->hw_geom.heads;
 
 	if (!ped_alignment_init (&start_align, 0, cylinder_size))
 		return NULL;
@@ -796,14 +802,16 @@ pc98_alloc_metadata (PedDisk* disk)
 {
 	PedPartition*		new_part;
 	PedConstraint*		constraint_any = NULL;
+	PedSector		cyl_size;
 
 	PED_ASSERT (disk != NULL, goto error);
 	PED_ASSERT (disk->dev != NULL, goto error);
 
 	constraint_any = ped_constraint_any (disk->dev);
 
+	cyl_size = disk->dev->hw_geom.sectors * disk->dev->hw_geom.heads;
 	new_part = ped_partition_new (disk, PED_PARTITION_METADATA, NULL,
-				0, disk->dev->sectors * disk->dev->heads - 1);
+				      0, cyl_size - 1);
 	if (!new_part)
 		goto error;
 
