@@ -184,19 +184,20 @@ sun_alloc (PedDevice* dev)
 	   defaults since most people do anyway...sue me.  */
 	label->magic	= PED_CPU_TO_BE16 (SUN_DISK_MAGIC);
 	label->nacyl	= PED_CPU_TO_BE16 (2);
-	label->pcylcount	= PED_CPU_TO_BE16 (dev->cylinders);
+	label->pcylcount	= PED_CPU_TO_BE16 (dev->hw_geom.cylinders);
 	label->rspeed	= PED_CPU_TO_BE16 (5400);
 	label->ilfact	= PED_CPU_TO_BE16 (1);
-	label->sparecyl	= PED_CPU_TO_BE16 (dev->sectors);
-	label->ntrks	= PED_CPU_TO_BE16 (dev->heads);
-	label->nsect	= PED_CPU_TO_BE16 (dev->sectors);
-	label->ncyl	= PED_CPU_TO_BE16 (dev->cylinders - 2);
+	label->sparecyl	= PED_CPU_TO_BE16 (dev->hw_geom.sectors);
+	label->ntrks	= PED_CPU_TO_BE16 (dev->hw_geom.heads);
+	label->nsect	= PED_CPU_TO_BE16 (dev->hw_geom.sectors);
+	label->ncyl	= PED_CPU_TO_BE16 (dev->hw_geom.cylinders - 2);
 
 	/* Add a whole disk partition at a minimum */
 	label->infos[WHOLE_DISK_PART].id = WHOLE_DISK_ID;
 	label->partitions[WHOLE_DISK_PART].start_cylinder = 0;
 	label->partitions[WHOLE_DISK_PART].num_sectors =
-		PED_CPU_TO_BE32((dev->cylinders-2) * dev->heads * dev->sectors);
+		PED_CPU_TO_BE32((dev->hw_geom.cylinders-2)
+			 * dev->hw_geom.heads * dev->hw_geom.sectors);
 
 	/* Now a neato string to describe this label */
 	snprintf(label->info, sizeof(label->info) - 1,
@@ -241,8 +242,12 @@ _check_geometry_sanity (PedDisk* disk, SunRawLabel* label)
 {
 	PedDevice*	dev = disk->dev;
 
-	if (PED_BE16_TO_CPU(label->nsect) != dev->sectors ||
-	    PED_BE16_TO_CPU(label->ntrks) != dev->heads) {
+	if (PED_BE16_TO_CPU(label->nsect) == dev->hw_geom.sectors &&
+	    PED_BE16_TO_CPU(label->ntrks) == dev->hw_geom.heads)
+		dev->bios_geom = dev->hw_geom;
+
+	if (PED_BE16_TO_CPU(label->nsect) != dev->bios_geom.sectors ||
+	    PED_BE16_TO_CPU(label->ntrks) != dev->bios_geom.heads) {
 #ifndef DISCOVER_ONLY
 		if (ped_exception_throw (
 				PED_EXCEPTION_WARNING,
@@ -250,18 +255,21 @@ _check_geometry_sanity (PedDisk* disk, SunRawLabel* label)
 				_("The disk CHS geometry (%d,%d,%d) does not "
 				  "match the geometry stored on the disk "
 				  "label (%d,%d,%d)."),
-				dev->cylinders, dev->heads, dev->sectors,
+				dev->hw_geom.cylinders,
+			       	dev->hw_geom.heads,
+			       	dev->hw_geom.sectors,
 				PED_BE16_TO_CPU(label->pcylcount),
 				PED_BE16_TO_CPU(label->ntrks),
 				PED_BE16_TO_CPU(label->nsect))
 			== PED_EXCEPTION_CANCEL)
 			return 0;
 #endif
-		dev->sectors = PED_BE16_TO_CPU(label->nsect);
-		dev->heads = PED_BE16_TO_CPU(label->ntrks);
-		dev->cylinders = PED_BE16_TO_CPU(label->pcylcount);
+		dev->bios_geom.sectors = PED_BE16_TO_CPU(label->nsect);
+		dev->bios_geom.heads = PED_BE16_TO_CPU(label->ntrks);
+		dev->bios_geom.cylinders = PED_BE16_TO_CPU(label->pcylcount);
 
-		if (dev->sectors * dev->heads * dev->cylinders > dev->length) {
+		if (dev->bios_geom.sectors * dev->bios_geom.heads
+			      	* dev->bios_geom.cylinders > dev->length) {
 			if (ped_exception_throw (
 				PED_EXCEPTION_WARNING,
 				PED_EXCEPTION_IGNORE_CANCEL,
@@ -300,8 +308,8 @@ sun_read (PedDisk* disk)
 	if (!_check_geometry_sanity (disk, label))
 		goto error;
 
-	block = disk->dev->sectors * disk->dev->heads;
-	disk_data->length = block * (disk->dev->cylinders
+	block = disk->dev->bios_geom.sectors * disk->dev->bios_geom.heads;
+	disk_data->length = block * (disk->dev->bios_geom.cylinders
 					- PED_BE16_TO_CPU(label->nacyl));
 
 	for (i = 0; i < SUN_DISK_MAXPARTITIONS; i++) {
@@ -354,7 +362,7 @@ _probe_and_add_use_old_info (PedDisk* disk)
 	if (!ped_device_read (disk->dev, &old_label, 0, 1))
 		return;
 	if (old_label.info [0]
-	    && old_label.magic == PED_CPU_TO_LE32 (SUN_DISK_MAGIC))
+	    && PED_BE16_TO_CPU (old_label.magic) == SUN_DISK_MAGIC)
 		memcpy (&sun_specific->raw_label, &old_label, 512);
 }
 
@@ -402,7 +410,8 @@ sun_write (PedDisk* disk)
 		label->infos[i].id = sun_data->type;
 		label->partitions[i].start_cylinder
 			= PED_CPU_TO_BE32 (part->geom.start
-				/ (disk->dev->sectors * disk->dev->heads));
+				/ (disk->dev->bios_geom.sectors
+				       	* disk->dev->bios_geom.heads));
 		label->partitions[i].num_sectors
 			= PED_CPU_TO_BE32 (part->geom.end
 					   - part->geom.start + 1);
@@ -416,8 +425,8 @@ sun_write (PedDisk* disk)
 	   smaller than it really is, but we'll have that problem even if we
 	   don't do this.  */
 
-	label->pcylcount = PED_CPU_TO_BE16 (disk->dev->cylinders);
-	label->ncyl = PED_CPU_TO_BE16 (disk->dev->cylinders
+	label->pcylcount = PED_CPU_TO_BE16 (disk->dev->bios_geom.cylinders);
+	label->ncyl = PED_CPU_TO_BE16 (disk->dev->bios_geom.cylinders
 	       				- PED_BE32_TO_CPU(label->nacyl));
 
 	sun_compute_checksum (label);
@@ -620,7 +629,7 @@ _get_strict_constraint (PedDisk* disk)
         PedAlignment    end_align;
         PedGeometry     max_geom;
 	SunDiskData*	disk_data = disk->disk_specific;
-	PedSector	block = dev->sectors * dev->heads;
+	PedSector	block = dev->bios_geom.sectors * dev->bios_geom.heads;
 
         if (!ped_alignment_init (&start_align, 0, block))
                 return NULL;
@@ -640,7 +649,7 @@ _get_lax_constraint (PedDisk* disk)
         PedAlignment    start_align;
         PedGeometry     max_geom;
 	SunDiskData*	disk_data = disk->disk_specific;
-	PedSector	block = dev->sectors * dev->heads;
+	PedSector	block = dev->bios_geom.sectors * dev->bios_geom.heads;
 
         if (!ped_alignment_init (&start_align, 0, block))
                 return NULL;
