@@ -606,6 +606,23 @@ int ext2_resize_fs(struct ext2_fs *fs, blk_t newsize, PedTimer* timer)
 		return 0;
 	}
 
+	if (EXT2_SUPER_FEATURE_COMPAT(fs->sb)
+			& EXT2_FEATURE_COMPAT_HAS_DIR_INDEX) {
+		if (ped_exception_throw (
+			PED_EXCEPTION_WARNING, PED_EXCEPTION_IGNORE_CANCEL,
+			_("The file system has the 'dir_index' feature "
+			  "enabled.  Parted can only resize the file system "
+			  "if it disables this feature.  You can enable it "
+			  "later by running 'tune2fs -O dir_index DEVICE' "
+			  "and then 'e2fsck -fD DEVICE'."))
+				!= PED_EXCEPTION_IGNORE)
+			return 0;
+		fs->sb.s_feature_compat
+			= PED_CPU_TO_LE32(EXT2_SUPER_FEATURE_COMPAT(fs->sb)
+					  & ~EXT2_FEATURE_COMPAT_HAS_DIR_INDEX);
+		fs->metadirty |= EXT2_META_SB;
+	}
+
 	if (fs->opt_verbose)
 		fprintf(stderr, "ext2_resize_fs\n");
 
