@@ -63,6 +63,8 @@ _device_get_length (PedDevice* dev)
 static int
 _device_probe_geometry (PedDevice* dev)
 {
+	PedSector cyl_size;
+
 	dev->length = _device_get_length (dev);
 	if (!dev->length)
 		return 0;
@@ -72,12 +74,13 @@ _device_probe_geometry (PedDevice* dev)
 		return 0;
 
 	/* XXX: We have no way to get this!  */
-	dev->sectors = 32;
-	dev->heads = 4;
-	dev->geom_known = 0;
-	dev->cylinders = dev->length / (dev->heads * dev->sectors
-					* (dev->sector_size / PED_SECTOR_SIZE));
-	
+	dev->bios_geom.sectors = 63;
+	dev->bios_geom.heads = 255;
+	cyl_size = dev->bios_geom.sectors * dev->bios_geom.heads;
+	dev->bios_geom.cylinders = dev->length / cyl_size
+					* (dev->sector_size / PED_SECTOR_SIZE);
+	dev->hw_geom = dev->bios_geom;
+
 	return 1;
 }
 
@@ -175,8 +178,6 @@ _init_device (const char *path)
 	dev->external_mode = 0;
 	dev->dirty = 0;
 	dev->boot_dirty = 0;
-	dev->geom_known = 1;
-	dev->geom_already_guessed = 0;
 
 	return dev;
 
@@ -733,13 +734,15 @@ gnu_check (PedDevice* dev, void* buffer, PedSector start, PedSector count)
 static int
 gnu_sync (PedDevice* dev)
 {
-	GNUSpecific*		arch_specific = GNU_SPECIFIC (dev);
+	GNUSpecific*		arch_specific;
 	error_t			err;
 	PedExceptionOption	ex_status;
 	static char *last_failure = NULL;
 
 	PED_ASSERT (dev != NULL, return 0);
 	PED_ASSERT (!dev->external_mode, return 0);
+	
+	arch_specific = GNU_SPECIFIC (dev);
 
 	if (dev->read_only || ! dev->dirty)
 		return 1;
