@@ -1,6 +1,6 @@
 /*
     libparted - a library for manipulating disk partitions
-    Copyright (C) 1999, 2000, 2001, 2002 Free Software Foundation, Inc.
+    Copyright (C) 1999, 2000, 2001, 2002, 2004 Free Software Foundation, Inc.
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -61,10 +61,10 @@
 #define HDIO_GET_IDENTITY	0x030d	/* get IDE identification info */
 
 struct hd_geometry {
-      unsigned char heads;
-      unsigned char sectors;
-      unsigned short cylinders;
-      unsigned long start;
+	unsigned char heads;
+	unsigned char sectors;
+	unsigned short cylinders;
+	unsigned long start;
 };
 
 /* structure returned by HDIO_GET_IDENTITY, as per ANSI ATA2 rev.2f spec */
@@ -376,7 +376,7 @@ _device_get_sector_size (PedDevice* dev)
 				== PED_EXCEPTION_IGNORE)
 			return sector_size;
 		else
-			return 0;
+			return PED_SECTOR_SIZE;
 	}
 
 	return sector_size;
@@ -405,7 +405,7 @@ _device_get_length (PedDevice* dev)
 
         if (_kernel_has_blkgetsize64()) {
                 if (ioctl(arch_specific->fd, BLKGETSIZE64, &bytes) == 0) {
-                        return bytes / _device_get_sector_size(dev);
+                        return bytes / PED_SECTOR_SIZE;
 		}
 	}
 
@@ -430,43 +430,39 @@ _device_probe_geometry (PedDevice* dev)
 	struct hd_geometry	geometry;
 
 	if (!_device_stat (dev, &dev_stat))
-		goto error;
-	PED_ASSERT (S_ISBLK (dev_stat.st_mode), goto error);
+		return 0;
+	PED_ASSERT (S_ISBLK (dev_stat.st_mode), return 0);
 
 	dev->length = _device_get_length (dev);
 	if (!dev->length)
-		goto error;
-
-	if (ioctl (arch_specific->fd, HDIO_GETGEO, &geometry)) {
-		ped_exception_throw (PED_EXCEPTION_ERROR,
-				     PED_EXCEPTION_CANCEL,
-				     _("Could not read geometry of %s - %s."),
-				     dev->path, strerror (errno));
-		goto error;
-	}
+		return 0;
 
 	dev->sector_size = _device_get_sector_size (dev);
 	if (!dev->sector_size)
-		goto error;
+		return 0;
 
-	dev->sectors = geometry.sectors;
-	dev->heads = geometry.heads;
-	if (!dev->sectors || !dev->heads)
-		goto error_dodgey_geometry;
+	/* The GETGEO ioctl is no longer useful (as of linux 2.6.x).  We could
+	 * still use it in 2.4.x, but this is contentious.  Perhaps we should
+	 * move to EDD. */
+	dev->bios_geom.sectors = 63;
+	dev->bios_geom.heads = 255;
+	dev->bios_geom.cylinders
+	       	= dev->length / (63 * 255)
+	       		/ (dev->sector_size / PED_SECTOR_SIZE);
 
-	dev->cylinders = dev->length / (dev->heads * dev->sectors
-					* (dev->sector_size / PED_SECTOR_SIZE));
+	/* FIXME: what should we put here?  (TODO: discuss on linux-kernel) */
+	if (!ioctl (arch_specific->fd, HDIO_GETGEO, &geometry)) {
+		dev->hw_geom.sectors = geometry.sectors;
+		dev->hw_geom.heads = geometry.heads;
+		dev->hw_geom.cylinders
+		       	= dev->length / (dev->hw_geom.heads
+				         * dev->hw_geom.sectors)
+				/ (dev->sector_size / PED_SECTOR_SIZE);
+	} else {
+		dev->hw_geom = dev->bios_geom;
+	}
 
 	return 1;
-
-error_dodgey_geometry:
-	ped_exception_throw (
-		PED_EXCEPTION_ERROR,
-		PED_EXCEPTION_CANCEL,
-		_("Device %s has dodgey geometry."),
-		dev->path);
-error:
-	return 0;
 }
 
 static char*
@@ -685,11 +681,11 @@ init_file (PedDevice* dev)
 
 	ped_device_close (dev);
 
-	dev->cylinders = dev->length / 4 / 32;
-	dev->heads = 4;
-	dev->sectors = 32;
+	dev->bios_geom.cylinders = dev->length / 4 / 32;
+	dev->bios_geom.heads = 4;
+	dev->bios_geom.sectors = 32;
+	dev->hw_geom = dev->bios_geom;
 	dev->sector_size = 512;
-	dev->geom_known = 0;
 	dev->model = strdup ("");
 	return 1;
 
@@ -737,11 +733,10 @@ init_generic (PedDevice* dev, char* model_name)
 
 		/* what should we stick in here? */
 		dev->length = dev_stat.st_size / PED_SECTOR_SIZE;
-		dev->cylinders = dev->length / 4 / 32;
-		dev->heads = 4;
-		dev->sectors = 32;
+		dev->bios_geom.cylinders = dev->length / 4 / 32;
+		dev->bios_geom.heads = 4;
+		dev->bios_geom.sectors = 32;
 		dev->sector_size = PED_SECTOR_SIZE;
-		dev->geom_known = 0;
 	}
 
 	dev->model = strdup (model_name);
@@ -780,8 +775,6 @@ linux_new (const char* path)
 	dev->external_mode = 0;
 	dev->dirty = 0;
 	dev->boot_dirty = 0;
-	dev->geom_known = 1;
-	dev->geom_already_guessed = 0;
 
 	if (!_device_probe_type (dev))
 		goto error_free_arch_specific;
@@ -809,27 +802,20 @@ linux_new (const char* path)
 
 	case PED_DEVICE_ATARAID:
 		if (!init_generic (dev, _("ATARAID Controller")))
-			goto error_free_dev;
+			goto error_free_arch_specific;
 		break;
 
 	case PED_DEVICE_I2O:
 		if (!init_generic (dev, _("I2O Controller")))
-			goto error_free_dev;
+			goto error_free_arch_specific;
 		break;
 
  	case PED_DEVICE_FILE:
 		if (!init_file (dev))
-			goto error_free_dev;
+			goto error_free_arch_specific;
 		break;
 
 	case PED_DEVICE_UNKNOWN:
-		if (ped_exception_throw (
-			PED_EXCEPTION_WARNING,
-			PED_EXCEPTION_IGNORE_CANCEL,
-			_("Device %s is neither a SCSI nor IDE drive."),
-			dev->path)
-				== PED_EXCEPTION_CANCEL)
-			goto error_free_arch_specific;
 		if (!init_generic (dev, _("Unknown")))
 			goto error_free_arch_specific;
 		break;
@@ -867,11 +853,16 @@ linux_is_busy (PedDevice* dev)
 	int	i;
 	char*	part_name;
 
+	if (_partition_is_mounted_by_path (dev->path))
+		return 1;
+
 	for (i = 0; i < 32; i++) {
 		int status;
 
 		part_name = _device_get_part_path (dev, i);
-		status = _partition_is_mounted_by_path (part_name);			
+		if (!part_name)
+			return 1;
+		status = _partition_is_mounted_by_path (part_name);
 		ped_free (part_name);
 
 		if (status)
@@ -1011,10 +1002,12 @@ llseek (unsigned int fd, loff_t offset, unsigned int whence)
 static int
 _device_seek (PedDevice* dev, PedSector sector)
 {
-	LinuxSpecific*	arch_specific = LINUX_SPECIFIC (dev);
+	LinuxSpecific*	arch_specific;
 
 	PED_ASSERT (dev != NULL, return 0);
 	PED_ASSERT (!dev->external_mode, return 0);
+	
+	arch_specific = LINUX_SPECIFIC (dev);
 
 #if SIZEOF_OFF_T < 8
 	if (sizeof (off_t) < 8) {
@@ -1031,11 +1024,13 @@ _device_seek (PedDevice* dev, PedSector sector)
 static int
 _read_lastoddsector (PedDevice* dev, void* buffer)
 {
-	LinuxSpecific*			arch_specific = LINUX_SPECIFIC (dev);
+	LinuxSpecific*			arch_specific;
 	struct blkdev_ioctl_param	ioctl_param;
 
 	PED_ASSERT(dev != NULL, return 0);
 	PED_ASSERT(buffer != NULL, return 0);
+	
+	arch_specific = LINUX_SPECIFIC (dev);
 
 retry:
 	ioctl_param.block = 0; /* read the last sector */
@@ -1067,14 +1062,17 @@ linux_read (PedDevice* dev, void* buffer, PedSector start, PedSector count)
 	PedExceptionOption	ex_status;
 	size_t			read_length = count * PED_SECTOR_SIZE;
 
-	/* Kludge.  This is necessary to read/write the last
-	   block of an odd-sized disk, until Linux 2.5.x kernel fixes.
-	*/
-	if (dev->type != PED_DEVICE_FILE && (dev->length & 1)
-	    && start + count - 1 == dev->length - 1)
-		return ped_device_read (dev, buffer, start, count - 1)
-			&& _read_lastoddsector (dev, buffer + (count-1) * 512);
 
+	if (_get_linux_version() < KERNEL_VERSION (2,6,0)) {
+		/* Kludge.  This is necessary to read/write the last
+		   block of an odd-sized disk, until Linux 2.5.x kernel fixes.
+		*/
+		if (dev->type != PED_DEVICE_FILE && (dev->length & 1)
+		    && start + count - 1 == dev->length - 1)
+			return ped_device_read (dev, buffer, start, count - 1)
+				&& _read_lastoddsector (
+					dev, buffer + (count-1) * 512);
+	}
 	while (1) {
 		if (_device_seek (dev, start))
 			break;
@@ -1135,11 +1133,13 @@ linux_read (PedDevice* dev, void* buffer, PedSector start, PedSector count)
 static int
 _write_lastoddsector (PedDevice* dev, const void* buffer)
 {
-	LinuxSpecific*			arch_specific = LINUX_SPECIFIC (dev);
+	LinuxSpecific*			arch_specific;
 	struct blkdev_ioctl_param	ioctl_param;
 
 	PED_ASSERT(dev != NULL, return 0);
 	PED_ASSERT(buffer != NULL, return 0);
+	
+	arch_specific = LINUX_SPECIFIC (dev);
 
 retry:
 	ioctl_param.block = 0; /* write the last sector */
@@ -1184,14 +1184,16 @@ linux_write (PedDevice* dev, const void* buffer, PedSector start,
 			return 1;
 	}
 
-	/* Kludge.  This is necessary to read/write the last
-	   block of an odd-sized disk, until Linux 2.5.x kernel fixes.
-	*/
-	if (dev->type != PED_DEVICE_FILE && (dev->length & 1)
-	    && start + count - 1 == dev->length - 1)
-		return ped_device_write (dev, buffer, start, count - 1)
-			&& _write_lastoddsector (dev, buffer + (count-1) * 512);
-
+	if (_get_linux_version() < KERNEL_VERSION (2,6,0)) {
+		/* Kludge.  This is necessary to read/write the last
+		   block of an odd-sized disk, until Linux 2.5.x kernel fixes.
+		*/
+		if (dev->type != PED_DEVICE_FILE && (dev->length & 1)
+		    && start + count - 1 == dev->length - 1)
+			return ped_device_write (dev, buffer, start, count - 1)
+				&& _write_lastoddsector (
+					dev, buffer + (count-1) * 512);
+	}
 	while (1) {
 		if (_device_seek (dev, start))
 			break;
@@ -1258,7 +1260,7 @@ static PedSector
 linux_check (PedDevice* dev, void* buffer, PedSector start, PedSector count)
 {
 	LinuxSpecific*	arch_specific = LINUX_SPECIFIC (dev);
-	int		done = 0;
+	PedSector	done = 0;
 	int		status;
 
 	if (!_device_seek (dev, start))
@@ -1266,7 +1268,7 @@ linux_check (PedDevice* dev, void* buffer, PedSector start, PedSector count)
 
 	for (done = 0; done < count; done += status / PED_SECTOR_SIZE) {
 		status = read (arch_specific->fd, buffer,
-			       (count-done) * PED_SECTOR_SIZE);
+			       (size_t) ((count-done) * PED_SECTOR_SIZE));
 		if (status < 0)
 			break;
 	}
@@ -1467,136 +1469,57 @@ linux_partition_get_path (const PedPartition* part)
 	return _device_get_part_path (part->disk->dev, part->num);
 }
 
-/* FIXME: should this be part of the API?  Should it report a warning if the
- * stat fails? */
 static int
-_partition_is_root_device (const char* part_name)
+_mount_table_search (const char* file_name, dev_t dev)
 {
-	unsigned int	root_dev;
-	struct stat	part_stat;
-	FILE*		proc_real_root_dev;
+	struct stat part_stat;
+	char line[512];
+	char part_name[512];
+	FILE* file;
 
-	/* even though this is an error, it probably means the partition
-	 * doesn't exist, etc., and therefore isn't root ;)
-	 */
-	if (stat (part_name, &part_stat))
+	file = fopen (file_name, "r");
+	if (!file)
 		return 0;
-
-	proc_real_root_dev = fopen ("/proc/sys/kernel/real-root-dev", "r");
- 	if (!proc_real_root_dev)
- 		goto error;
- 	fscanf (proc_real_root_dev, "%d", &root_dev);
- 	fclose (proc_real_root_dev);
- 	return root_dev == part_stat.st_rdev;
-
-error:
-	return -1;
-}
-
-/* returns:
- * 	1  == mounted
- * 	0  == not mounted
- * 	-1 == if unknown
- *
- * NOTE: /proc/mounts doesn't list the real root device (in Linux), but
- * /dev/root instead.  Since the root device is always mounted (by
- * definition), this is checked separately in _partition_is_mounted_by_path()
- */
-static int
-_mount_table_search (const char* mtab_path, const char* part_name)
-{
-	char		line [256];
-	FILE*		mtab;
-
-	mtab = fopen (mtab_path, "r");
-	if (!mtab)
-		goto error;
-
-	while (1) {
-		if (!fgets (line, 256, mtab)) {
-			if (feof (mtab)) {
-				fclose (mtab);
-				return 0;
-			} else {
-				goto error_close_mtab;
+ 	while (fgets (line, 512, file)) {
+		sscanf (line, "%s", part_name);
+		if (stat (part_name, &part_stat) == 0) {
+			if (part_stat.st_rdev == dev) {
+				fclose (file);
+				return 1;
 			}
 		}
-		if (strncmp (line, part_name, strlen (part_name)) == 0
-		    && isspace (line [strlen (part_name)])) {
-			fclose (mtab);
-			return 1;
-		}
-	}
-
-error_close_mtab:
-	fclose (mtab);
-error:
-	ped_exception_throw (
-		PED_EXCEPTION_WARNING,
-		PED_EXCEPTION_IGNORE,
-		_("Error reading %s (%s) to determine if partition is "
-		  "mounted."),
-		mtab_path, strerror (errno));
-
-	return -1;
-}
-
-static int
-_partition_is_mounted_by_path (const char* path)
-{
-	static int can_probe = 1;
-
-	if (!can_probe)
-		return 0;
-
-	switch (_partition_is_root_device (path)) {
-		case 1: return 1;
-		case -1: goto unknown;
-	}
-	switch (_mount_table_search ("/proc/mounts", path)) {
-		case 1: return 1;
-		case -1:
-			switch (_mount_table_search ("/etc/mtab", path)) {
-				case 1: return 1;
-				case -1: goto unknown;
-			}
-	}
-	switch (_mount_table_search ("/proc/swaps", path)) {
-		case 1: return 1;
-		case -1: goto unknown;
-	}
+ 	}
+ 	fclose (file);
 	return 0;
-
-unknown:
-	can_probe = 0;
-	return ped_exception_throw (
-			PED_EXCEPTION_WARNING,
-			PED_EXCEPTION_IGNORE_CANCEL,
-			_("Unable to determine if partitions are mounted via "
-			  "/proc/mounts or /etc/mtab.  Make sure you don't "
-			  "attempt to resize or modify mounted file systems.  "
-			  "(Even read-only mounted)"))
-		!= PED_EXCEPTION_IGNORE;
 }
 
 static int
-_partition_is_mounted (const PedPartition* part)
+_partition_is_mounted_by_path (const char *path)
 {
-	char*		part_name;
-	int		status;
-
-	PED_ASSERT (part != NULL, return 0);
-
-	if (part->num == -1)
+	struct stat part_stat;
+	if (stat (path, &part_stat) != 0)
 		return 0;
+	if (!S_ISBLK(part_stat.st_mode))
+		return 0;
+	return  _mount_table_search( "/proc/mounts", part_stat.st_rdev)
+		|| _mount_table_search( "/proc/swaps", part_stat.st_rdev)
+		|| _mount_table_search( "/etc/mtab", part_stat.st_rdev);
+}
 
+static int
+_partition_is_mounted (const PedPartition *part)
+{
+	char* part_name;
+	int result;
+
+	if (!ped_partition_is_active (part))
+		return 0;
 	part_name = _device_get_part_path (part->geom.dev, part->num);
 	if (!part_name)
-		return 0;
-
-	status = _partition_is_mounted_by_path (part_name);
+		return 1;
+	result = _partition_is_mounted_by_path (part_name);
 	ped_free (part_name);
-	return status;
+	return result;
 }
 
 static int
@@ -1608,16 +1531,12 @@ linux_partition_is_busy (const PedPartition* part)
 
 	if (_partition_is_mounted (part))
 		return 1;
-
 	if (part->type == PED_PARTITION_EXTENDED) {
 		for (walk = part->part_list; walk; walk = walk->next) {
-			if (!ped_partition_is_active (walk))
-				continue;
 			if (linux_partition_is_busy (walk))
 				return 1;
 		}
 	}
-
 	return 0;
 }
 
